﻿Imports System.IO.MemoryMappedFiles
Imports System.IO

Public Class WaveReader
    Private header As WaveHeader
    Private m_format As WaveFormat
    Private m_data As WaveData
    Private m_filename As String
    Private mmf As MemoryMappedFile

    Public Sub New()
        initFields()
        readWaveFromFile()
    End Sub

    Public ReadOnly Property Data() As WaveData
        Get
            Return m_data
        End Get
    End Property
    Public ReadOnly Property Format() As WaveFormat
        Get
            Return m_format
        End Get
    End Property

    Public Sub New(filename As String)
        initFields()
        Me.m_filename = filename
        readWaveFromFile()
    End Sub

    Private Sub initFields()
        header = New WaveHeader()
        m_format = New WaveFormat()
        m_data = New WaveData()
        m_filename = ""
    End Sub

    Public Property Filename() As String
        Get
            Return Me.m_filename
        End Get
        Set(value As String)
            Me.m_filename = value
        End Set
    End Property

    Public ReadOnly Property File() As MemoryMappedFile
        Get
            Return mmf
        End Get
    End Property

    Private Sub readWaveFromFile()
        mmf = MemoryMappedFile.CreateFromFile(m_filename, FileMode.Open)
        Using viewAccessor = mmf.CreateViewAccessor(0, 44)
            Dim bytes As Byte() = New Byte(3) {}
            viewAccessor.ReadArray(Of Byte)(0, bytes, 0, 4)
            header.RiffType = System.Text.Encoding.[Default].GetString(bytes)
            header.FileLength = viewAccessor.ReadUInt32(4)
            viewAccessor.ReadArray(Of Byte)(8, bytes, 0, 4)
            header.GroupID = System.Text.Encoding.[Default].GetString(bytes)
            viewAccessor.ReadArray(Of Byte)(12, bytes, 0, 4)
            m_format.FormatID = System.Text.Encoding.[Default].GetString(bytes)
            m_format.Size = viewAccessor.ReadUInt32(16)
            m_format.FormatTag = viewAccessor.ReadUInt16(20)
            m_format.Channels = viewAccessor.ReadUInt16(22)
            m_format.SamplesPerSecond = viewAccessor.ReadUInt32(24)
            m_format.AverageBytesPerSecond = viewAccessor.ReadUInt32(28)
            m_format.BlockAlign = viewAccessor.ReadUInt16(32)
            m_format.BitsPerSample = viewAccessor.ReadUInt16(34)
            viewAccessor.ReadArray(Of Byte)(36, bytes, 0, 4)
            m_data.DataID = System.Text.Encoding.[Default].GetString(bytes)
            m_data.Size = viewAccessor.ReadUInt32(40)

            Debug.Write(header.ToString() + m_format.ToString() + m_data.ToString())
        End Using
    End Sub

    Public ReadOnly Property Duration() As UInteger
        Get
            Return m_data.Size \ m_format.AverageBytesPerSecond
        End Get
    End Property
End Class



Friend Class WaveHeader
    Private sGroupID As String
    ' RIFF
    Private dwFileLength As UInteger
    ' total file length minus 8, which is taken up by RIFF
    Private sRiffType As String
    ' always WAVE
    ''' <summary>
    ''' Initializes a WaveHeader object with the default values.
    ''' </summary>
    Public Sub New()
        dwFileLength = 0
        sGroupID = "RIFF"
        sRiffType = "WAVE"
    End Sub

    Public Property GroupID() As String
        Get
            Return sGroupID
        End Get
        Set(value As String)
            sGroupID = value
        End Set
    End Property

    Public Property FileLength() As UInteger
        Get
            Return dwFileLength
        End Get
        Set(value As UInteger)
            dwFileLength = value
        End Set
    End Property

    Public Property RiffType() As String
        Get
            Return sRiffType
        End Get
        Set(value As String)
            sRiffType = value
        End Set
    End Property

    Public Overrides Function ToString() As String
        Return String.Format("WAVE HEADER" & vbCr & vbLf & "Group ID: {0}" & vbCr & vbLf & "File Length: {1}" & vbCr & vbLf & "Riff Type: {2}" & vbCr & vbLf & vbCr & vbLf, sGroupID, dwFileLength, sRiffType)
    End Function
End Class



Public Class WaveFormat
    Private sChunkID As String
    ' Four bytes: "fmt "
    Private dwChunkSize As UInteger
    ' Length of header in bytes
    Private wFormatTag As UShort
    ' 1 (MS PCM)
    Private wChannels As UShort
    ' Number of channels
    Private dwSamplesPerSec As UInteger
    ' Frequency of the audio in Hz... 44100
    Private dwAvgBytesPerSec As UInteger
    ' for estimating RAM allocation
    Private wBlockAlign As UShort
    ' sample frame size, in bytes
    Private wBitsPerSample As UShort
    ' bits per sample
    Public Property FormatID() As String
        Get
            Return sChunkID
        End Get
        Set(value As String)
            sChunkID = value
        End Set
    End Property
    Public Property Size() As UInteger
        Get
            Return dwChunkSize
        End Get
        Set(value As UInteger)
            dwChunkSize = value
        End Set
    End Property
    Public Property FormatTag() As UShort
        Get
            Return wFormatTag
        End Get
        Set(value As UShort)
            wFormatTag = value
        End Set
    End Property
    Public Property Channels() As UShort
        Get
            Return wChannels
        End Get
        Set(value As UShort)
            wChannels = value
        End Set
    End Property
    Public Property SamplesPerSecond() As UInteger
        Get
            Return dwSamplesPerSec
        End Get
        Set(value As UInteger)
            dwSamplesPerSec = value
        End Set
    End Property
    Public Property AverageBytesPerSecond() As UInteger
        Get
            Return dwAvgBytesPerSec
        End Get
        Set(value As UInteger)
            dwAvgBytesPerSec = value
        End Set
    End Property
    Public Property BlockAlign() As UShort
        Get
            Return wBlockAlign
        End Get
        Set(value As UShort)
            wBlockAlign = value
        End Set
    End Property
    Public Property BitsPerSample() As UShort
        Get
            Return wBitsPerSample
        End Get
        Set(value As UShort)
            wBitsPerSample = value
        End Set
    End Property
    ''' <summary>
    ''' Initializes a format chunk with the following properties:
    ''' Sample rate: 44100 Hz
    ''' Channels: Stereo
    ''' Bit depth: 16-bit
    ''' </summary>
    Public Sub New()
        sChunkID = "fmt "
        dwChunkSize = 16
        wFormatTag = 1
        wChannels = 2
        dwSamplesPerSec = 44100
        wBitsPerSample = 16
        wBlockAlign = CUShort(wChannels * (wBitsPerSample \ 8))
        dwAvgBytesPerSec = dwSamplesPerSec * wBlockAlign
    End Sub
    Public Overrides Function ToString() As String
        Return String.Format("WAVE FORMAT" & vbCr & vbLf & "ID: {0}" & vbCr & vbLf & "Size: {1}" & vbCr & vbLf & "Format Tag: {2}" & vbCr & vbLf & "Channels: {3}" & vbCr & vbLf & "Samples Per Second: {4}" & vbCr & vbLf & "Bits Per Sample: {5}" & vbCr & vbLf & "Block Align: {6}" & vbCr & vbLf & "Average Bytes Per Second: {7}" & vbCr & vbLf & vbCr & vbLf, sChunkID, dwChunkSize, wFormatTag, wChannels, dwSamplesPerSec, _
         wBitsPerSample, wBlockAlign, dwAvgBytesPerSec)
    End Function
End Class



Public Class WaveData
    Private sChunkID As String
    ' "data"
    Private dwChunkSize As UInteger
    ' Length of header in bytes
    Public Property DataID() As String
        Get
            Return sChunkID
        End Get
        Set(value As String)
            sChunkID = value
        End Set
    End Property
    Public Property Size() As UInteger
        Get
            Return dwChunkSize
        End Get
        Set(value As UInteger)
            dwChunkSize = value
        End Set
    End Property

    ''' <summary>
    ''' Initializes a new data chunk with default values.
    ''' </summary>
    Public Sub New()
        dwChunkSize = 0
        sChunkID = "data"
    End Sub


    Public Overloads Function ToString() As String
        Return String.Format("WAVE DATA" & vbCr & vbLf & "ID: {0}" & vbCr & vbLf & "Size: {1}" & vbCr & vbLf, sChunkID, dwChunkSize)
    End Function
End Class
