﻿Imports System.ComponentModel
Imports MindFusion.RealTimeCharting.Wpf
Imports System.Media
Imports System.Windows.Threading

Class MainWindow
    Inherits Window
    Implements INotifyPropertyChanged

    Private reader As WaveReader
    Private audioSerie As Series
    Private position As Series
    Private startPos As Point
    Private endPos As Point
    Private yAxis As Axis
    Private yAxisRight As Axis
    Private bytes As Long
    Private seconds As Integer
    Private index As Integer
    Private filename As String
    Private player As SoundPlayer
    Private startTime As DateTime
    Private dispatcherTimer As System.Windows.Threading.DispatcherTimer
    Private lineTimer As System.Windows.Threading.DispatcherTimer
    Public _loadedPoints As ULong


    Public Sub New()

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        init()
        musicGraphic.YAxisCollection.Add(yAxis)
        musicGraphic.Y2AxisCollection.Add(yAxisRight)
        lineTimer = New System.Windows.Threading.DispatcherTimer(DispatcherPriority.Send)
    End Sub

    Private Sub init()
        seconds = 0
        bytes = 44
        index = 0
        yAxis = New Axis()
        yAxis.Origin = -1
        yAxis.Length = 2
        yAxis.Interval = 0.2
        yAxisRight = New Axis()
        yAxisRight.Origin = -1
        yAxisRight.Length = 2
        yAxisRight.Interval = 0.2
        yAxisRight.LabelHorizontalAlignment = HorizontalAlignment.Right
        audioSerie = New Series(yAxis)
        startPos = New Point(0, -1)
        endPos = New Point(0, 1)
        filename = ""
        _loadedPoints = 0
        musicGraphic.XAxis = New Axis()
        musicGraphic.XAxis.Title = "Time in seconds"
    End Sub

    Public Property LoadedPoints() As ULong
        Get
            Return _loadedPoints
        End Get
        Set(value As ULong)
            _loadedPoints = value
            OnPropertyChanged("LoadedPoints")
        End Set
    End Property

    Private Sub OnLoaded(sender As Object, e As RoutedEventArgs)
        musicGraphic.OnApplyTemplate()
        Dim a As Axis = musicGraphic.XAxis
        musicGraphic.IndicatorLine.Y2 = musicGraphic.PlotArea.ActualHeight - (a.LabelFontSize + a.TickLength + 2 * a.LabelOffset + (If(a.Title IsNot Nothing AndAlso a.Title <> "", (a.TitleFontSize + a.TitleOffset), 0)))
        loadFile("thats_it.wav")
        'downloaded from http://www.nichecom.com/songs/
    End Sub

    Private Sub AddGraphics(sender As Object, e As EventArgs)
        Dim onePointBytes As Integer = reader.Format.BitsPerSample / 8 * reader.Format.Channels
        Dim pointsInMB As Integer = 1024 * 1024 \ onePointBytes
        Dim bytesPerSecond As Integer = CInt(reader.Data.Size) \ CInt(reader.Duration)
        If seconds < reader.Duration Then
            Using viewAccessor = reader.File.CreateViewAccessor(bytes, bytesPerSecond)
                Dim old As Integer = index
                Dim i As Integer = 0
                While i < reader.Format.SamplesPerSecond
                    If reader.Format.BitsPerSample = 8 Then
                        Dim pointY As Double = (CDbl(viewAccessor.ReadByte(i)) - 128.0) / 128.0
                        Dim pointX As Double = CDbl(seconds) + CDbl(i) / reader.Format.SamplesPerSecond
                        Dim toAdd As New Point(pointX, pointY)
                        audioSerie.Data.Add(toAdd)
                    End If
                    If reader.Format.BitsPerSample = 16 Then
                        Dim pointY As Double = CDbl(viewAccessor.ReadInt16(i)) / 42000.0
                        Dim pointX As Double = CDbl(seconds) + CDbl(i) / reader.Format.SamplesPerSecond
                        Dim toAdd As New Point(pointX, pointY)
                        audioSerie.Data.Add(toAdd)
                    End If
                    If reader.Format.BitsPerSample = 32 Then
                        Dim pointY As Double = CDbl(viewAccessor.ReadSingle(i))
                        Dim pointX As Double = CDbl(seconds) + CDbl(i) / reader.Format.SamplesPerSecond
                        Dim toAdd As New Point(pointX, pointY)
                        audioSerie.Data.Add(toAdd)
                    End If
                    LoadedPoints += 1
                    index += 1
                    i += 3 * CInt(reader.Duration) * reader.Format.Channels
                End While
                musicGraphic.Commit(old)
                seconds += 1
            End Using
            bytes += bytesPerSecond
        Else
            reader.File.Dispose()
            player = New SoundPlayer(filename)
            dispatcherTimer.[Stop]()
            Play.IsEnabled = True
        End If
    End Sub

    Private Sub DrawPosition(sender As Object, e As EventArgs)
        Dim currentTime As TimeSpan = DateTime.Now - startTime
        Dim offset As Double = CDbl(musicGraphic.LineCanvas.ActualWidth) * (currentTime.TotalSeconds / (CDbl(reader.Data.Size) / CDbl(reader.Format.AverageBytesPerSecond)))
        If musicGraphic.IndicatorLine.X1 < musicGraphic.LineCanvas.ActualWidth Then
            musicGraphic.IndicatorLine.X1 = offset
            musicGraphic.IndicatorLine.X2 = offset
        End If
        If musicGraphic.IndicatorLine.X1 > musicGraphic.LineCanvas.ActualWidth Then
            musicGraphic.IndicatorLine.X1 = musicGraphic.LineCanvas.ActualWidth
            musicGraphic.IndicatorLine.X2 = musicGraphic.LineCanvas.ActualWidth
            lineTimer.[Stop]()
            [Stop].IsEnabled = False
            Play.IsEnabled = True
        End If
    End Sub

    Private Sub Play_Click(sender As Object, e As RoutedEventArgs)
        If filename IsNot Nothing AndAlso filename <> "" Then
            musicGraphic.IndicatorLine.X1 = 0
            musicGraphic.IndicatorLine.X2 = 0
            player.Play()
            startTime = DateTime.Now
            lineTimer = New System.Windows.Threading.DispatcherTimer(DispatcherPriority.Send)
            AddHandler lineTimer.Tick, AddressOf DrawPosition
            'lineTimer.Tick += New EventHandler(AddressOf DrawPosition)
            lineTimer.Interval = New TimeSpan(0, 0, 0, 0, 100)
            audioSerie.Stroke = New SolidColorBrush(Colors.Red)
            lineTimer.Start()
            Play.IsEnabled = False
            [Stop].IsEnabled = True
        End If
    End Sub

    Private Sub Stop_Click(sender As Object, e As RoutedEventArgs)
        player.[Stop]()
        lineTimer.[Stop]()
        [Stop].IsEnabled = False
        Play.IsEnabled = True
    End Sub

    Private Sub Browse_Click(sender As Object, e As RoutedEventArgs)
        init()
        'musicGraphic.ClearCache();
        Dim dlg As New Microsoft.Win32.OpenFileDialog()

        ' Set filter for file extension and default file extension
        dlg.DefaultExt = ".wav"
        dlg.Filter = "Wave (.wav)|*.wav"

        ' Display OpenFileDialog by calling ShowDialog method
        Dim result As Nullable(Of Boolean) = dlg.ShowDialog()

        ' Get the selected file name and display in a TextBox
        If result = True Then
            ' Open document
            'musicGraphic.FastScrollMode = true;
            filename = dlg.FileName
            tb2.Text = "File: " & filename

            loadFile(filename)
        End If
    End Sub

    Private Sub loadFile(filename As String)
        Me.filename = filename
        reader = New WaveReader(filename)

        musicGraphic.XAxis.Interval = reader.Duration / 10
        musicGraphic.XAxis.Length = reader.Duration

        If musicGraphic.SeriesCollection.Count = 0 Then
            musicGraphic.SeriesCollection.Add(audioSerie)
        Else
            audioSerie.Data.Clear()
            musicGraphic.SeriesCollection(0) = audioSerie
        End If

        musicGraphic.Commit()
        dispatcherTimer = New System.Windows.Threading.DispatcherTimer(DispatcherPriority.Send)
        AddHandler dispatcherTimer.Tick, AddressOf AddGraphics
        'dispatcherTimer.Tick += New EventHandler(AddGraphics)
        dispatcherTimer.Interval = TimeSpan.FromMilliseconds(10)
        dispatcherTimer.Start()
    End Sub

    Private Sub OnSizeChanged(sender As Object, e As SizeChangedEventArgs)
        Dim a As Axis = musicGraphic.XAxis
        musicGraphic.IndicatorLine.Y2 = musicGraphic.PlotArea.ActualHeight - (a.LabelFontSize + a.TickLength + 2 * a.LabelOffset + (If(a.Title IsNot Nothing AndAlso a.Title <> "", (a.TitleFontSize + a.TitleOffset), 0)))
    End Sub

    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

    '[NotifyPropertyChangedInvocator]
    Protected Overridable Sub OnPropertyChanged(propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub
End Class
